
ExcludeArch: %{ix86}

%define mpich_name mpich

%global blaslib openblas

Name:    ga
Version: 5.8.2
Release:    1%{?dist}
Summary: Global Arrays Toolkit
License: BSD
Source0: https://github.com/GlobalArrays/ga/releases/download/v%{version}/ga-%{version}.tar.gz
URL: http://github.com/GlobalArrays/ga
ExclusiveArch: %{ix86} x86_64 %{arm} aarch64 ppc64le 
BuildRequires: openmpi-devel, %{mpich_name}-devel, gcc-c++, gcc-gfortran
BuildRequires: %{blaslib}-devel, openssh-clients, dos2unix

%define ga_desc_base \
The Global Arrays (GA) toolkit provides an efficient and portable \
"shared-memory" programming interface for distributed-memory \
computers. Each process in a MIMD parallel program can asynchronously \
access logical blocks of physically distributed dense multi- \
dimensional arrays, without need for explicit cooperation by other \
processes. Unlike other shared-memory environments, the GA model \
exposes to the programmer the non-uniform memory access (NUMA) \
characteristics of the high performance computers and acknowledges \
that access to a remote portion of the shared data is slower than to \
the local portion. The locality information for the shared data is \
available, and a direct access to the local portions of shared data \
is provided.

%description
%{ga_desc_base}
- Global Arrays Toolkit Base Package.

%package common
Summary: Global Arrays Common Files
BuildArch: noarch
%description common
%{ga_desc_base}
- Global Arrays Common Files.

%package mpich
Summary: Global Arrays Toolkit for MPICH
BuildRequires: scalapack-%{mpich_name}-devel
BuildRequires: %{blaslib}-devel
Requires: %{name}-common = %{version}
Provides: %{name}-mpich2 = %{version}-%{release}
%description mpich
%{ga_desc_base}
- Libraries against MPICH.
%package mpich-devel
Summary: Global Arrays Toolkit for MPICH Development
Requires: scalapack-%{mpich_name}-devel, %{mpich_name}-devel
Requires: %{blaslib}-devel, %{name}-common = %{version}, %{name}-mpich = %{version}
Provides: %{name}-mpich2-devel = %{version}-%{release}
%description mpich-devel
%{ga_desc_base}
- Development Software against MPICH.
%package mpich-static
Summary: Global Arrays Toolkit for MPICH Static Libraries
Requires: scalapack-%{mpich_name}-devel, %{mpich_name}-devel
Requires: %{blaslib}-devel, %{name}-common = %{version}, %{name}-mpich = %{version}
Provides: %{name}-mpich2-static = %{version}-%{release}
%description mpich-static
%{ga_desc_base}
- Static Libraries against MPICH.

%package openmpi
Summary: Global Arrays Toolkit for OpenMPI
BuildRequires: scalapack-openmpi-devel
BuildRequires: %{blaslib}-devel
BuildRequires: make
Requires: %{name}-common = %{version}
%description openmpi
%{ga_desc_base}
- Libraries against OpenMPI.
%package openmpi-devel
Summary: Global Arrays Toolkit for OpenMPI Development
Requires: scalapack-openmpi-devel, openmpi-devel
Requires: %{blaslib}-devel, %{name}-common = %{version}, %{name}-openmpi = %{version}
%description openmpi-devel
%{ga_desc_base}
- Development Software against OpenMPI.
%package openmpi-static
Summary: Global Arrays Toolkit for OpenMPI Static Libraries
Requires: scalapack-openmpi-devel, openmpi-devel
Requires: %{blaslib}-devel, %{name}-common = %{version}, %{name}-openmpi = %{version}
%description openmpi-static
%{ga_desc_base}
- Static Libraries against OpenMPI.


%define ga_version %{version}

%prep
%setup -q -c -n %{name}-%{version}

pushd %{name}-%{ga_version}

popd
for i in mpich openmpi; do
  cp -a %{name}-%{ga_version} %{name}-%{version}-$i
done


%build
%define doBuild \
export LIBS="-lscalapack -l%{blaslib}" ; \
export CFLAGS="%{optflags} -O1" ; \
export CXXFLAGS="%{optflags} -O1" ; \
export FFLAGS="%{optflags} -O1" ; \
cd %{name}-%{version}-$MPI_COMPILER_NAME ; \
%configure \\\
  --bindir=$MPI_BIN \\\
  --libdir=$MPI_LIB \\\
  --includedir=$MPI_INCLUDE \\\
  --with-scalapack4=-lscalapack \\\
  --with-blas4=-l%{blaslib} \\\
  --enable-shared \\\
  --enable-static \\\
  --enable-cxx \\\
  --enable-f77 \\\
  $GA_CONFIGURE_OPTIONS ; \
%{__make} %{?__smp_mflags} ; \
cd ..

export MPI_COMPILER_NAME=mpich
export GA_CONFIGURE_OPTIONS=""
%{_mpich_load}
%doBuild
%{_mpich_unload}

export MPI_COMPILER_NAME=openmpi
%{_openmpi_load}
%doBuild
%{_openmpi_unload}


%install
%define doInstall \
cd %{name}-%{version}-$MPI_COMPILER_NAME ; \
DESTDIR=%{buildroot} make install ; \
cd ..

rm -rf %{buildroot}
export MPI_COMPILER_NAME=mpich
%{_mpich_load}
%doInstall
%{_mpich_unload}

export MPI_COMPILER_NAME=openmpi
%{_openmpi_load}
%doInstall
%{_openmpi_unload}

find %{buildroot} -type f -name "*.la" -exec rm -f {} \;

mkdir -p %{buildroot}/%{_sysconfdir}/sysctl.d
echo 'kernel.shmmax = 134217728' > %{buildroot}/%{_sysconfdir}/sysctl.d/armci.conf
%define do_test 1
%check
%if %{?do_test}0
%{_mpich_load}
cd %{name}-%{version}-mpich
make NPROCS=2 VERBOSE=1 check-ma check-travis
make NPROCS=2 TESTS="global/testing/test.x global/testing/testc.x global/testing/testmatmult.x global/testing/patch.x global/testing/simple_groups_comm.x global/testing/elempatch.x" check-TESTS VERBOSE=1
cd ..
%{_mpich_unload}
%{_openmpi_load}
cd %{name}-%{version}-openmpi
export OMPI_MCA_btl=^uct
export OMPI_MCA_btl_base_warn_component_unused=0
make NPROCS=2 VERBOSE=1 check-ma check-travis
make NPROCS=2 TESTS="global/testing/test.x global/testing/testc.x global/testing/testmatmult.x global/testing/patch.x global/testing/simple_groups_comm.x global/testing/elempatch.x" check-TESTS VERBOSE=1
cd ..
%{_openmpi_unload}
%endif

%files common
%doc %{name}-%{ga_version}/README.md %{name}-%{ga_version}/CHANGELOG.md
%doc %{name}-%{ga_version}/DISCLAIMER %{name}-%{ga_version}/LICENSE
%config(noreplace) %{_sysconfdir}/sysctl.d/armci.conf

%files mpich
%doc %{name}-%{ga_version}/DISCLAIMER %{name}-%{ga_version}/LICENSE
%{_libdir}/%{mpich_name}/lib/lib*.so.*
%{_libdir}/%{mpich_name}/bin/*.x
%files mpich-devel
%doc %{name}-%{ga_version}/DISCLAIMER %{name}-%{ga_version}/LICENSE
%{_libdir}/%{mpich_name}/lib/lib*.so
%{_includedir}/%{mpich_name}-%{_arch}/*
%{_libdir}/%{mpich_name}/bin/ga-config
%{_libdir}/%{mpich_name}/bin/armci-config
%{_libdir}/%{mpich_name}/bin/comex-config
%files mpich-static
%doc %{name}-%{ga_version}/DISCLAIMER %{name}-%{ga_version}/LICENSE
%{_libdir}/%{mpich_name}/lib/lib*.a

%files openmpi
%doc %{name}-%{ga_version}/DISCLAIMER %{name}-%{ga_version}/LICENSE
%{_libdir}/openmpi/lib/lib*.so.*
%{_libdir}/openmpi/bin/*.x
%files openmpi-devel
%doc %{name}-%{ga_version}/DISCLAIMER %{name}-%{ga_version}/LICENSE
%{_libdir}/openmpi/lib/lib*.so
%{_includedir}/openmpi-%{_arch}/*
%{_libdir}/openmpi/bin/ga-config
%{_libdir}/openmpi/bin/armci-config
%{_libdir}/openmpi/bin/comex-config
%files openmpi-static
%doc %{name}-%{ga_version}/DISCLAIMER %{name}-%{ga_version}/LICENSE
%{_libdir}/openmpi/lib/lib*.a

%changelog
* Fri Apr 12 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 5.8.2-1
- initial build
